#include "generatorcore.h"
#include "../AGBoxDog/boxshm.h"
#include "../agVideoProcess/boxdeviceshm.h"
AGBoxShm *agBoxShm;
BoxDeviceShm *boxDeviceShm;

void GeneratorCore::shm_init(){
    QSqlQuery qry;
    QString sql = QString("select mqtt_ip,mqtt_port,user_name,pass_word from yt_t_mqtt where item_name = 'data-agbox'");
    qry = db.exec(sql);
    while (qry.next()) {
        ip = qry.value(0).toString();
        port = qry.value(1).toString();
        username = qry.value(2).toString();
        password = qry.value(3).toString();
    }
    qry.clear();
    sql.clear();

}

bool GeneratorCore::shm_load(){
    key_t key;
    int shmid;
    if((key=ftok(SHM_PATH,static_cast<int>(SHM_PORT)))==-1){
        return false;
    }
    if((shmid=shmget(key,sizeof(AGBoxShm),IPC_CREAT|0666))==-1){
        return false;
    }
    agBoxShm = static_cast<AGBoxShm *>(shmat(shmid,nullptr,0));

    if((key=ftok(VIDEO_PATH,static_cast<int>(VIDEO_PORT)))==-1){
        return false;
    }
    if((shmid=shmget(key,sizeof(BoxDeviceShm),IPC_CREAT|0666))==-1){
        return false;
    }
    boxDeviceShm = static_cast<BoxDeviceShm *>(shmat(shmid,nullptr,0));

    shm_init();
    return true;
}

GeneratorCore::GeneratorCore(QObject *parent) : QObject(parent)
{
    logthread = new LogThread(this);
    logthread->start();

    db = QSqlDatabase::addDatabase("QSQLITE","conf_db");
    db.setDatabaseName(QString("/opt/db/yt_conf.db"));
    if(!db.open()){
        logthread->appendData(QString("[aggenerator] open yt_conf.db failed"));
    }
    if(shm_load()){
        logthread->appendData(QString("[aggenerator] shm load success"));
    }

    mqttIdx = 1;
    m_client = new QMQTT::Client(QHostAddress(ip),static_cast<quint16>(port.toInt()),this);
    connect(m_client,&QMQTT::Client::connected,this,&GeneratorCore::onConnected);
    m_client->setUsername(username);
    m_client->setPassword(password.toLatin1());
    m_client->setCleanSession(true);
    m_client->connectToHost();

    dbthread = new DatabaseThread(this);
    dbthread->start();

    httpthread = new HttpThread(this);
    connect(httpthread,&HttpThread::dataLog,this,&GeneratorCore::dataLog);
    connect(httpthread,&HttpThread::mqttData,this,&GeneratorCore::mqtt_data);
    connect(httpthread,&HttpThread::appendSql,this,&GeneratorCore::append_sql);

}


GeneratorCore::~GeneratorCore()
{
    logthread->stop();
    dbthread->stop();
    httpthread->stop();
}

void GeneratorCore::start()
{

}
void GeneratorCore::onConnected()
{
    logthread->appendData(QString("mqtt onConnected"));
}

void GeneratorCore::dataLog(QString log)
{
    logthread->appendData(log);
}

void GeneratorCore::mqtt_data(QString mqtt_msg)
{
    printf("test4444 [%s]\n",mqtt_msg.toUtf8().data());
    if((m_client->connectionState()==QMQTT::STATE_INIT)||(m_client->connectionState()==QMQTT::STATE_DISCONNECTED)){
        m_client->connectToHost();
    }
    m_client->publish(QMQTT::Message(mqttIdx++,"data-collector",mqtt_msg.toUtf8()));
    if(mqttIdx > 9999){
        mqttIdx = 1;
    }
}

void GeneratorCore::append_sql(QString sql)
{
    dbthread->appendSql(sql);
}




